<?PHP
/**Project K++
 *
 * Project K++ (nee "Canned Kandinsky"), an exploration of computer-generated
 * abstract visual art. Originally an attempt to simulate the work of Kandinsky
 * through programmatic means, the project has evolved into an emulation of his 
 * work, extrapolating from his interest in ever more abstract representations
 * into creating art through an abstract -- that is, mathematical -- medium: 
 * code.
 *
 * The project will provide a framework for exploring heuristics derived from 
 * aesthetic principles. The goal of the project is to create progressively more 
 * humanly "pleasing" or "artistic" works. The side effect, which is in this 
 * case more important than the goal, is a deeper understanding of what the 
 * words "artistic" and "pleasing" mean. This project is unique in that it 
 * attempts to improve our understanding of these words in a more quantifiable 
 * way.
 *
 * This release (0.1.1a) is purely a proof-of-concept release. What does it do?
 * Well, sometimes it makes pretty pictures. Sometimes - it's avant garde. ;)
 *
 * The improvements in the 0.1.1a release include the ability to drag any object
 * autonomously and the elimination of randomMorphs from the painting.
 *
 * @package CannedKandinsky
 * @author Robert Peake <robert@peakepro.com>
 * @copyright &copy; 2004
 * @link http://www.robertpeake.com/CannedKandinsky
 * @license http://www.php.net/license/3_0.txt
 * @version 0.1a (proof-of-concept)
*/
/**function safe_rand
 *
 * return a random integer between $i and $j
 * whether or not $i<$j 
 *
 * @access public
 * @param int $i
 * @param int $j
 * @return int
*/
function safe_rand($i,$j) {
  if($i<$j) return rand($i,$j);
  else if($i>$j) return rand($j,$i);
  else if($i==$j) return $i;
  else return FALSE;
}

/** class colorTheme
 *
 * A class to create colors in a theme
 *
 * @access public
 * @package CannedKandinsky
 *
*/
class colorTheme {
  public $num_colors = 1;
  private $r_min = 0x00;
  private $r_max = 0x00;
  private $g_min = 0x00;
  private $g_max = 0x00;
  private $b_min = 0x00;
  private $b_max = 0x00;
  private $a_min = 0x00;
  private $a_max = 0x00;
  private $swatch = array();
  private $previous_color = FALSE;
  private $INVERT = FALSE;

  function __construct() {
    $this->num_colors = safe_rand(1,5);
    $this->r_min = safe_rand(0x00,0xff);
    $this->r_max = safe_rand($r_min,0xff);
    $this->g_min = safe_rand(0x00,0xff);
    $this->g_max = safe_rand($g_min,0xff);
    $this->b_min = safe_rand(0x00,0xff);
    $this->b_max = safe_rand($b_min,0xff);
    $this->a_min = safe_rand(0x00,0xff);
    $this->a_max = safe_rand($a_min,0xff);
  }

/** function getColor
 *
 * Uses existing state information and a variety of subtle
 * color transforms to return an rgba color array within a
 * color theme.
 *
 * @access public
 * @return array
 *
*/
  public function getColor() {
    if(!$this->previous_color) {
      if(safe_rand(0,1)) {
       $color = array($this->r_min,$this->g_min,$this->b_min,$this->a_max);
      } else {
       $color = array($this->r_max,$this->g_max,$this->b_max,$this->a_min);
      }
    } else if(safe_rand(0,1)) {
      $color = $this->permute($this->previous_color); 
    }else if($this->num_colors > sizeof($this->swatch)) {
      $r = safe_rand($this->r_min,$this->r_max);
      $g = safe_rand($this->g_min,$this->g_max);
      $b = safe_rand($this->b_min,$this->b_max);
      $a = safe_rand($this->a_min,$this->a_max);
      $color = array($r,$g,$b,$a);
      $this->swatch[] = $color;
    } else {
      $color_i = safe_rand(0,(sizeof($this->swatch)-1));
      $color = $this->swatch[$color_i];
      if(safe_rand(0,1) && $this->INVERT) {
        $color = $this->inverse($color);
      } else {
        $color = $this->permute($color);
      }
    }
    $this->previous_color = $color;
    return $color;
  }

/** function getBackgroundColor
 *
 * Uses existing state information and the darker and mute
 * color transforms to return an rgba color array suitable
 * for use as a background color.
 *
 * @access public
 * @return array
*/
  public function getBackgroundColor() {
    $color = $this->getColor();
    $color = $this->darker($color);
    $color = $this->mute($color);
    $color = $this->darker($color);
    return $color;
  }

/** function inverse
 *
 * returns an inversion of the color array
 *
 * @access private
 * @param array $color
 * @return array $color
*/
  private function inverse($color) {
    return array( abs($color[0]-0xff), abs($color[1]-0xff), abs($color[2]-0xff), $color[3] );
  }

/** function lighter
 *
 *
 * returns a lighter version of the color array
 *
 * @access private
 * @param array $color
 * @return array
*/
  private function lighter($color) {
    $r = $color[0];
    $g = $color[1];
    $b = $color[2];
    $r2 = $r + 0x33;
    $g2 = $g + 0x33;
    $b2 = $b + 0x33;
    if($r2 > 0xff) $r2 = 0xff;
    if($g2 > 0xff) $g2 = 0xff;
    if($b2 > 0xff) $b2 = 0xff;
    return array($r2,$g2,$b2,$color[3]);
  }

/** function darker
 *
 * returns a darker version of the color array
 *
 * @access private
 * @param array $color
 * @return array
*/
  private function darker($color) {
    $r = $color[0];
    $g = $color[1];
    $b = $color[2];
    $r2 = $r - 0x33;
    $g2 = $g - 0x33;
    $b2 = $b - 0x33;
    if($r2 < 0x00) $r2 = 0x00;
    if($g2 < 0x00) $g2 = 0x00;
    if($b2 < 0x00) $b2 = 0x00;
    return array($r2,$g2,$b2,$color[3]);
  }

/** function transparent
 *
 * returns a more transparent version of the color array
 *
 * @access private
 * @param array $color
 * @return array
*/
  private function transparent($color) {
    $a = $color[3];
    if($a < 0x33) $a2 = $a;
    else {
    $a2 = $a - 0x33;
    if($a2 < 0x33) $a2 = 0x33;
    } return array($color[0],$color[1],$color[2],$a2);
  }

/** function is_white
 *
 * returns true if given an array representing white
 *
 * @access private
 * @param array $color
 * @return bool
*/
  private function is_white($color) {
    if($color[0] == 0xff && $color[1] == 0xff && $color[2] == 0xff)
      return TRUE;
    else
      return FALSE;
  }

/** function is_black
 *
 * returns true if given an array representing black
 *
 * @access private
 * @param array $color
 * @return bool
*/
  private function is_black($color) {
    if($color[0] == 0x00 && $color[1] == 0x00 && $color[2] == 0x00)
      return TRUE;
    else
      return FALSE;
  }

/** function mute
 *
 * returns a more muted version of the color array
 * where each component is increased or decreased halfway
 * toward an average of all three colors (n)
 *
 * @access private
 * @param array $color
 * @return array
*/
  private function mute($color) {
    $r = $color[0];
    $g = $color[1];
    $b = $color[2];
    $n = ($r + $g + $b) / 3; //n is the average
    $r2 = ($r + $n)/2;
    $g2 = ($g + $n)/2;
    $b2 = ($b + $n)/2;
    return array($r2,$g2,$b2,$color[3]);
  }

/** function permute
 *
 * randomly applies one of: transparent, lighter, darker or mute
 * if the resultant color is pure white or pure black, the function
 * increases the transparency to make the color more interesting
 *
 * @access private
 * @param array $color
 * @return array
*/
  private function permute($color) {
    $switch = safe_rand(0,3);
    switch($switch) {
      case 0:
      $color = $this->transparent($color);
      break;
      case 1:
      $color = $this->lighter($color);
      break;
      case 2:
      $color = $this->darker($color);
      break;
      case 3:
      $color = $this->mute($color);
      break;
      default:
      break;
    }
    if($this->is_white($color) || $this->is_black($color)) $color = $this->transparent($this->previous_color);
    return $color;
  }

}

/** class randomCircle
 *
 * A class to create a circle with radius
 * between 0 and 9
 *
 * @access public
 * @package CannedKandinsky
*/
class randomCircle extends SWFShape {

  private $sides;
  private $color;
  private $color2;

/** function __construct
 *
 * creates a randomCircle with optional radial gradient fill
 *
 * @access public
 * @param array $color
 * @param array $color2
 * @param bool $GRADIENT
*/
  function __construct ($color=array(0x00,0x00,0x00,0xff),$color2=array(0xff,0xff,0xff,0x00),$GRADIENT=FALSE) {
    parent::__construct(); //instantiate all the SWFShape goodness - Ming needs this
    $this->color = $color;
    $this->color2 = $color2;
    if($GRADIENT) { //create a gradient fill using both colors
      $g = new SWFGradient();
      $g->addEntry($color[0],$color[1],$color[2],$color[3]);
      $g->addEntry($color2[0],$color2[1],$color2[2],$color[3]);
      $f = $this->addFill($g, SWFFILL_RADIAL_GRADIENT);
      $f->scaleTo(1/safe_rand(10,100));
      $f->moveTo(safe_rand(0,9),safe_rand(0,9));
    } else { //create a normal fill using the first color
      $f = $this->addFill($color[0],$color[1],$color[2],$color[3]);
    }
    $this->setRightFill($f);
    $r = rand(0,9);
    $this->movePen(0,-$r);
    $c = ($r/(1+sqrt(2)));
    $d = ($r/(2+sqrt(2)));
    $this->drawCurve($c, 0, $d, $d);
    $this->drawCurve($d, $d, 0, $c);
    $this->drawCurve(0, $c, -$d, $d);
    $this->drawCurve(-$d, $d, -$c, 0);
    $this->drawCurve(-$c, 0, -$d, -$d);
    $this->drawCurve(-$d, -$d, 0, -$c);
    $this->drawCurve(0, -$c, $d, -$d);
    $this->drawCurve($d, -$d, $c, 0);
  }

}

/** class randomShape
 *
 * This polygon will have 3-9 sides
 *
 * @access public
 * @package CannedKandinsky
 *
 * A class to create a random pologon with optional linear gradient fill
 * and optional use of curves (yes by default)
*/
class randomShape extends SWFShape {

  private $sides;
  private $color;
  private $color2;

/** function __construct
 *
 * creates a randomShape with optional use of curves (yes by default) and linear
 * gradient fill (no by default as it looks pretty strange)
 *
 * @access public
 * @param array $color
 * @param array $color2
 * @param bool $CURVES
 * @param bool $GRADIENT
*/
  function __construct($color=array(0x00,0x00,0x00,0xff),$color2=array(0xff,0xff,0xff,0x00),$CURVES=1,$GRADIENT=0) {
    parent::__construct(); //instantiate SWFShape goodness (Ming needs this)
    $this->color = $color;
    $this->color2 = $color2;
    $this->sides = safe_rand(3,9);
    if($GRADIENT) { //use a linear gradient fill
      $g = new SWFGradient();
      $g->addEntry($color[0],$color[1],$color[2],$color[3]);
      $g->addEntry($color2[0],$color2[1],$color2[2],$color[3]);
      $f = $this->addFill($g, SWFFILL_LINEAR_GRADIENT);
      $f->scaleTo(1/safe_rand(100,1000));
      $f->moveTo(safe_rand(0,9),safe_rand(0,9));
    } else { //use a solid fill
      $f = $this->addFill($color[0],$color[1],$color[2],$color[3]);
    }
    $this->setRightFill($f);
    $this->movePenTo(0,0); //start at origin
    $initial_x = safe_rand(0,9);
    $initial_y = safe_rand(0,9);
    $initial_cx = safe_rand(0,9);
    $initial_cy = safe_rand(0,9);

    //flip a coin
    if(safe_rand(0,1) && $CURVES) { 
    //if curves are allowed and we win the coin toss, draw a curve
      $this->drawCurve($initial_cx,$initial_cy,$initial_x,$initial_y);
    } else { //otherwise, draw a plain line
      $this->drawLine($initial_x,$initial_y);
    }
    $this->sides -= 1; //decrement the number of sides we need to draw
    while($this->sides >= 1) { //while there are still sides left to draw
     //create our four new values in such a way that we still
     //satisfy a right-fill (clockwise) shape
     $next_x = safe_rand(-$initial_x,$initial_x);
     $next_y = safe_rand(0,-$initial_y);
     $next_cx = safe_rand(-$initial_cx,$initial_cx);
     $next_cy = safe_rand(0,-$initial_cy);
     //another coin toss
     if( safe_rand(0,1) && CURVES) {
       $this->drawCurve($next_cx,$next_cy,$next_x,$next_y);
     } else {
       $this->drawLine($next_x,$next_y);
     }  
     $this->sides -= 1;
    }
    $this->drawLineTo(0,0); //return to origin
  }
}

/** class randomShape
 *
 * A class to create clusters of 3, 5, or 7 random pologons, each 
 * with optional linear gradient fills
 * and optional use of curves (yes by default)
 *
 * @access public
 * @package CannedKandinsky
 *
 * Each polygon will have 3-9 sides. Polygons are arranged vertically.
*/
class randomSymphonic extends SWFSprite {
    
  private $sides = array();
  private $color = array();
  private $shapes = array();

/** function __construct
 *
 * creates a randomSymphonic with optional use of curves (yes by default) and
 * linear gradient fills (no by default) and optional forced number of elements
 *
 * @access public
 * @param array $color
 * @param array $color2
 * @param bool $CURVES
 * @param bool $GRADIENT
*/
  function __construct ($color=array(0x00,0x00,0x00,0xff),$color2=array(0xff,0xff,0xff,0x00),$CURVES=1,$GRADIENT=0,$myname='0',$ELEMENTS=FALSE) {
    parent::__construct(); //instantiate SWFSprite goodness (Ming needs this)
    $this->setFrames(1);

    $this->color[] = $color;
    $this->color[] = $color2;
    $this->sides[] = safe_rand(3,9);

    $coin = safe_rand(0,2);
    if(!$ELEMENTS) if($coin==0) $ELEMENTS = 3; else if($coin==1) $ELEMENTS = 5; else $ELEMENTS = 7;
    while($ELEMENTS-- > 0) { //while there are elements left to draw
      $s = 's'.$ELEMENTS; //the element variable name
      $$s = new SWFShape(); //the element variable itself
      /**
       * Why not just use a randomShape you ask? Because these symphonics
       * are permutations of the same shape (i.e. an SWFShape drawn with the
       * same parameters). The permutations include skewing and rotating, but
       * the shapes themselves should be discernably related.
      */
      if($GRADIENT) {
        $g = new SWFGradient();
        $g->addEntry($color[0],$color[1],$color[2],$color[3]);
        $g->addEntry($color2[0],$color2[1],$color2[2],$color2[3]);
        $f = $$s->addFill($g, SWFFILL_LINEAR_GRADIENT);
        $f->scaleTo(1/safe_rand(100,1000));
        $f->moveTo(safe_rand(0,9),safe_rand(0,9));
      } else {
        $f = $$s->addFill($color[0],$color[1],$color[2],$color[3]);
      }
      $$s->setRightFill($f);
      $$s->movePenTo(0,0); //start at origin
      $initial_x = safe_rand(0,9);
      $initial_y = safe_rand(0,9);
      $initial_cx = safe_rand(0,9);
      $initial_cy = safe_rand(0,9);
  
      if($CURVES && safe_rand(0,1)) {
        $$s->drawCurve($initial_cx,$initial_cy,$initial_x,$initial_y);
      } else {
        $$s->drawLine($initial_x,$initial_y);
      } 
      $$s->sides -= 1;
      while($$s->sides >= 1) {
       $next_x = safe_rand(-$initial_x,$initial_x);
       $next_y = safe_rand(0,-$initial_y);
       $next_cx = safe_rand(-$initial_cx,$initial_cx);
       $next_cy = safe_rand(0,-$initial_cy);
       if($CURVES && safe_rand(0,1)) {
         $$s->drawCurve($next_cx,$next_cy,$next_x,$next_y);
       } else {
         $$s->drawLine($next_x,$next_y);
       } 
       $$s->sides -= 1;
      }
      $$s->drawLineTo(0,0);
      $i = 'i'.$ELEMENTS;
	  $$b = new SWFButton();
      $$b->addShape($$s, SWFBUTTON_HIT | SWFBUTTON_UP | SWFBUTTON_DOWN | SWFBUTTON_OVER);
      $$b->addAction(new SWFAction("stop();startDrag(\"/$myname\", 0);"), SWFBUTTON_MOUSEDOWN);
      $$b->addAction(new SWFAction("play();stopDrag();"), SWFBUTTON_MOUSEUP | SWFBUTTON_MOUSEUPOUTSIDE);
      $$i = $this->add($$b);
	  $$i->setName($i);
      $$i->move(0,safe_rand(0,9));
      $this->shapes[] = $$i;
    }
    $this->nextFrame(); //when we're all done, we increment the frame
    //this is an SWFSprite-specific requirement to display properly.
  }
}

// End class declarations - begin sample of use
//First, some constants
$MAX_X = 770;
$MAX_Y = 500;
$FRAMES = safe_rand(240,480);
$curves = 1;

//instantiating the movie
$m = new SWFMovie();

//how many shapes?
$numshapes = safe_rand(3,10);

//create a colorTheme which we will use for everything
$ct = new colorTheme();
$bg_array = $ct->getBackgroundColor();
$r = $bg_array[0];
$g = $bg_array[1];
$b = $bg_array[2];

//set up the movie
$m->setDimension($MAX_X, $MAX_Y);
$m->setBackground($r,$g,$b);

//tell the colorTheme to use as many colors as there are
//actual shapes before repeating colors
$ct->num_colors = $numshapes;

//draw those shapes
while($numshapes-- > 0) { //while there are shapes left to draw
  $p = 'p'.$numshapes; //variable name
  $i = 'i'.$numshapes; //variable name
  $b = 'b'.$numshapes;
  $color = $ct->getColor();
  $color2 = $ct->getColor();
  $color3 = $ct->getColor();
  $color4 = $ct->getColor();
  $coin = safe_rand(1,3);
 if($coin==1) { //create a shape (button, actually, inside a sprite)
    $$p = new randomShape($color,$color2,$curves,rand(0,1));
    $$b = new SWFButton(); //wrap the shape in a button
    /**
     * Here we add some actionscript to make our buttons (which look
     * like randomShape objects) draggable
    */
    $$b->addShape($$p, SWFBUTTON_HIT | SWFBUTTON_UP | SWFBUTTON_DOWN | SWFBUTTON_OVER);
    $$b->addAction(new SWFAction("stop();startDrag(\"/$i\", 0);"), SWFBUTTON_MOUSEDOWN);
    $$b->addAction(new SWFAction("play();stopDrag();"), SWFBUTTON_MOUSEUP | SWFBUTTON_MOUSEUPOUTSIDE);
	$$s = new SWFSprite();
	$$s->add($$b);
	$$s->nextFrame();
    $$i = $m->add($$s);
    $$i->setName($i);
    $shape_arr[] = $$i; //add it to the shape array for later
  } else if($coin==2) { // add a symphonic cluster
    $$p = new randomSymphonic($color,$color2,$curves,rand(0,1),$i); //<-note we pass in the object name here as $i
    $$i = $m->add($$p);
    $$i->setName($i);
    $sprite_arr[] = $$i; //add the symphonic to the array for later
  } else if ($coin==3) { //circle (actually a button)
    $$p = new randomCircle($color1,$color2,rand(0,1));
    $$b = new SWFButton(); //wrap the circle in a button
    /**
     * Here we add some actionscript to make our buttons (which look
     * like randomCircle objects) handles to drag the movie
    */
    $$b->addShape($$p, SWFBUTTON_HIT | SWFBUTTON_UP | SWFBUTTON_DOWN | SWFBUTTON_OVER);
    $$b->addAction(new SWFAction("stop();startDrag(\"/$i\", 0);"), SWFBUTTON_MOUSEDOWN);
    $$b->addAction(new SWFAction("play();stopDrag();"), SWFBUTTON_MOUSEUP | SWFBUTTON_MOUSEUPOUTSIDE);
    $$s = new SWFSprite();
	$$s->add($$b);
	$$s->nextFrame();
    $$i = $m->add($$s);
    $$i->setName($i);
	$circle_arr[] = $$i; //add the circle to the array for later
  }
  
  /**
   * Here we play with the positioning. In later versions, we'll use the
   * arrays we just created to map hueristic transformations to the various
   * shapes for purposes of later analyzing the effectiveness. For now, we 
   * just scatter the shapes randomly on the canvass.
  */
  
  $$i->scale($MAX_X/(safe_rand(3,6)*9),$MAX_Y/(safe_rand(3,6)*9));
  $move_x = safe_rand(0,$MAX_X/safe_rand(1,3));
  $move_y = safe_rand(0,$MAX_Y/safe_rand(1,3));
  $$i->move($move_x,$move_y);
}

$m->nextFrame(); //add that last frame

header('Content-type: application/x-shockwave-flash'); //get ready, here comes an SWF
$m->output(); //and away we go!
?>